-------------------------------------------------------------------------------
--|> CROSSHAIR
--: Places a crosshair on a nameplate.
--: Filtered for player targets that are hostile player with support for icons, text etc attachments
-------------------------------------------------------------------------------
--|> INIT
-------------------------------------------------------------------------------
local weizPVP = weizPVP
local RC = LibStub("LibRangeCheck-2.0")
weizPVP.Crosshair = {}
local Crosshair = weizPVP.Crosshair
local CrosshairFrame

-------------------------------------------------------------------------------
--|> Upvalue Globals
-------------------------------------------------------------------------------
local RAID_CLASS_COLORS = RAID_CLASS_COLORS
local UIFrameFadeIn = UIFrameFadeIn
local Clamp = Clamp
local IsItemInRange = IsItemInRange
local UnitIsUnit = UnitIsUnit
local C_NamePlate = C_NamePlate
local UnitClass = UnitClass
local UnitIsConnected = UnitIsConnected
local IsFlying = IsFlying
local IsUsableItem = IsUsableItem
local select = select
local XEN = XEN

-------------------------------------------------------------------------------
--|> LOCALS
-------------------------------------------------------------------------------
--> Targeted Player Info
local targetNamePlateFrame
local targetPlayerName
local targetClass
local targetClassColor

--> Hide Crosshair
---------------------------------------
local function HideCrosshair()
    weizPVP_CrosshairFrame:Hide()
end
--> Show Crosshair
---------------------------------------
local function ShowCrosshair()
    UIFrameFadeIn(weizPVP_CrosshairFrame, 0.1, 0, weizPVP.Options.Crosshair.Alpha)
    CrosshairFrame:Show()
end

--> Active Target Timer
---------------------------------------
local timer_ActiveTarget
timer_ActiveTarget =
    C_Timer.NewTicker(
    1,
    function()
        timer_ActiveTarget:Cancel()
    end
)
timer_ActiveTarget:Cancel()

--> CONFIGURE RANGE
---------------------------------------
local function ConfigureRange()
    if weizPVP.Options.Crosshair.ShowRange then
        CrosshairFrame.RangeText:Show()
    else
        CrosshairFrame.RangeText:Hide()
    end
end

-->  SET CLASS COLORS
local function SetCrosshairColors(r, g, b)
    if not (r and g and b) then --> Get colors for class of targeted player if none provied
        r, g, b = targetClassColor:GetRGB()
    end
    --> CORE
    CrosshairFrame.Core:SetVertexColor(XEN.AdjustRGBTint(r, g, b, 0.2))
    CrosshairFrame.Core_ADD:SetVertexColor(XEN.AdjustRGBTint(r, g, b, 0.5))
    --> LINES
    CrosshairFrame.TopLine:SetVertexColor(XEN.AdjustRGBTint(r, g, b, 0.2))
    CrosshairFrame.BottomLine:SetVertexColor(XEN.AdjustRGBTint(r, g, b, 0.2))
    CrosshairFrame.LeftLine:SetVertexColor(XEN.AdjustRGBTint(r, g, b, 0.2))
    CrosshairFrame.RightLine:SetVertexColor(XEN.AdjustRGBTint(r, g, b, 0.2))
    --> FX
    CrosshairFrame.TargetFX:SetVertexColor(r, g, b, 0.8)
    --> ALPHA
    weizPVP.Crosshair.SetAlpha()
    --> NAME TEXT
    if weizPVP.Options.Crosshair.NameEnabled then
        CrosshairFrame.NameText:SetTextColor(r, g, b)
    end
end

-->  RANGE CHECK
local minRange, _
local outputString, outputAlpha
local function rangeCheck()
    minRange, _ = RC:GetRange("target")
    outputString, outputAlpha = "", 0
    if minRange then
        outputString = minRange
        --> Get min yards #
        if outputString >= 40 then -- solid @ 40+ yards
            outputAlpha = 1
        elseif minRange < 20 then
            outputAlpha = 0
        elseif minRange < 30 then
            outputAlpha = 0.5
        elseif minRange < 40 then
            outputAlpha = 0.8
        end
    end
    if outputString == 200 then
        outputString = "200+"
    end
    return outputString, outputAlpha
end

--> NET
local function ShowNet5000()
    CrosshairFrame.NetOMatic.BG.Pulse:Play()
    CrosshairFrame.NetOMatic:SetAlpha(1)
end

local function HideNet5000()
    CrosshairFrame.NetOMatic.BG.Pulse:Stop()
    CrosshairFrame.NetOMatic:SetAlpha(0)
end

local function IsNetUsable()
    return (IsFlying("target") and IsItemInRange("Net-o-Matic 5000", "target") and IsUsableItem("Net-o-Matic 5000"))
end

--> Target Changed
local function TargetRangeCheck()
    if targetNamePlateFrame then
        --> Net-o-Matic 5000 Check
        if IsNetUsable() or weizPVP.Options.DEBUG then
            ShowNet5000()
        else
            HideNet5000()
        end
        --> Range Check
        local rtext, ralpha = rangeCheck()
        if CrosshairFrame.RangeText:GetText() ~= rtext then
            CrosshairFrame.RangeText:SetText(rtext)
        end
        if CrosshairFrame.RangeText:GetAlpha() ~= ralpha then
            CrosshairFrame.RangeText:SetAlpha(ralpha)
        end
        if UnitIsConnected("target") == true then
            SetCrosshairColors()
        else
            SetCrosshairColors(0.3, 0.3, 0.3)
        end
    else
        if timer_ActiveTarget then
            timer_ActiveTarget:Cancel()
            timer_ActiveTarget = nil
        end
    end
end

--> Check KOS
-------------------------------------------------
local function CheckKOS()
    if weizPVP.KosList[weizPVP.GetUnitName("target")] then
        CrosshairFrame.FourArrowsKOS.FadeIn:Play()
        CrosshairFrame.FourArrowsKOS.Rotate:Play()
        CrosshairFrame.FourArrows:Hide()
        CrosshairFrame.FourArrowsKOS:Show()
        return true
    else
        CrosshairFrame.FourArrows.FadeIn:Play()
        CrosshairFrame.FourArrows.Rotate:Play()
        CrosshairFrame.FourArrows:Show()
        CrosshairFrame.FourArrowsKOS:Hide()
        return false
    end
end

--> Setup Crosshair On Nameplate
-------------------------------------------------
local function SetupCrosshairOnNameplate(nameplate)
    if not nameplate then
        return
    end
    --> Set on same level as nameplate
    CrosshairFrame.Core:SetSize(nameplate:GetWidth() + 14, nameplate:GetWidth() + 14)
    CrosshairFrame:ClearAllPoints()
    CrosshairFrame:SetParent(nameplate)
    CrosshairFrame:SetPoint("CENTER")
    --> Alpha
    if not CrosshairFrame:IsShown() then
        ShowCrosshair()
        weizPVP.Crosshair.SetAlpha()
        ShowCrosshair()
    end
    --> Animations
    CrosshairFrame:StopAnimating()
    CrosshairFrame.TargetFX:Show()
    CrosshairFrame.TargetFX.Splash:Play()
    --> KOS Check
    CheckKOS()
    --> APPLY COLORS
    if UnitIsConnected("target") == true then
        SetCrosshairColors()
    else
        SetCrosshairColors(DISABLED_FONT_COLOR:GetRGB())
    end
end

--> Refresh Name Text
-------------------------------------------------
local function RefreshNameText()
    if weizPVP.Options.Crosshair.NameEnabled then
        if strlen(targetPlayerName) > strlen(gsub(targetPlayerName, "-(.*)", "")) then
            --> From another server
            CrosshairFrame.NameText:SetText(targetClassColor:WrapTextInColorCode(gsub(targetPlayerName, "-(.*)", "")) .. "|cFFFF00CC*|r")
        else
            --> From same server
            CrosshairFrame.NameText:SetText(targetClassColor:WrapTextInColorCode(gsub(targetPlayerName, "-(.*)", "")))
        end
        CrosshairFrame.NameText:Show()
    else
        CrosshairFrame.NameText:Hide()
    end
end

--> Refresh Guild Text
-------------------------------------------------
local function RefreshGuildText()
    if weizPVP.Options.Crosshair.GuildEnabled then
        if select(1, GetGuildInfo("target")) then
            CrosshairFrame.GuildText:SetText("|cffcccccc" .. (select(1, GetGuildInfo("target")) or "") .. "|r")
            CrosshairFrame.GuildGradient:Show()
            CrosshairFrame.GuildTextBG:Show()
            CrosshairFrame.GuildText:Show()
        else
            CrosshairFrame.GuildGradient:Hide()
            CrosshairFrame.GuildTextBG:Hide()
            CrosshairFrame.GuildText:Hide()
        end
    else
        CrosshairFrame.GuildGradient:Hide()
        CrosshairFrame.GuildText:Hide()
        CrosshairFrame.GuildTextBG:Hide()
    end
end

--> IS TARGET VALID?
-------------------------------------------------
local function IsTargetValid()
    if (UnitCanAttack("player", "target") or weizPVP.Options.DEBUG) and UnitIsPlayer("target") then
        return true
    else
        return false
    end
end

--> VALID UNIT TARGETED
-------------------------------------------------------
local function ValidUnitTargeted()
    --> Update targt
    targetPlayerName = weizPVP.GetUnitName("target")
    targetClass = select(2, UnitClass("target"))
    targetClassColor = RAID_CLASS_COLORS[targetClass]
    --> Refresh text widgets for target
    RefreshNameText()
    RefreshGuildText()
    --> Check for nameplate
    targetNamePlateFrame = C_NamePlate.GetNamePlateForUnit("target") and true or nil
    if targetNamePlateFrame then
        ShowCrosshair()
        SetupCrosshairOnNameplate(C_NamePlate.GetNamePlateForUnit("target"))
    else
        HideCrosshair()
        return
    end
end

--> INVALID UNIT TARGETED
-------------------------------------------------------
local function InvalidUnitTargeted()
    weizPVP.Crosshair.Reset()
end

-------------------------------------------------------------------------------
--|> GLOBAL EVENT FUNCTIONS
-------------------------------------------------------------------------------

-->  NAMEPLATE ADDED
-------------------------------------------------------
function Crosshair.NameplateAdded(unit)
    if UnitIsUnit("target", unit) then --> Confirm this is the unit we want
        if not IsTargetValid() then
            return
        end
        --> Get player info
        targetPlayerName = weizPVP.GetUnitName("target")
        targetClass = select(2, UnitClass("target"))
        targetClassColor = RAID_CLASS_COLORS[targetClass]
        --> Refresh text widgets
        RefreshNameText()
        RefreshGuildText()
        ShowCrosshair()
        SetupCrosshairOnNameplate(C_NamePlate.GetNamePlateForUnit("target"))
        Crosshair.NewTarget()
    end
end

--> NAMEPLATE REMOVED
-------------------------------------------------------
function Crosshair.NameplateRemoved(unit)
    if UnitIsUnit("target", unit) then --> Confirm the target nameplate was removed
        Crosshair.NewTarget()
    end
end

--> NEW TARGET
-------------------------------------------------------
function Crosshair.NewTarget()
    if IsTargetValid() then
        ValidUnitTargeted()
    else
        InvalidUnitTargeted()
    end
end

-------------------------------------------------------------------------------
--|> FUNCTIONS: SETTINGS + CONFIG
-------------------------------------------------------------------------------
--> CROSSHAIR ALPHA
-------------------------------------------------------
function weizPVP.Crosshair.SetAlpha()
    CrosshairFrame:SetAlpha(weizPVP.Options.Crosshair.Alpha)
    CrosshairFrame.TopLine:SetAlpha(Clamp(weizPVP.Options.Crosshair.Alpha, 0, 0.6))
    CrosshairFrame.BottomLine:SetAlpha(Clamp(weizPVP.Options.Crosshair.Alpha, 0, 0.6))
    CrosshairFrame.LeftLine:SetAlpha(Clamp(weizPVP.Options.Crosshair.Alpha, 0, 0.6))
    CrosshairFrame.RightLine:SetAlpha(Clamp(weizPVP.Options.Crosshair.Alpha, 0, 0.6))
end

--> CHANGE LINE THICKNESS
-------------------------------------------------------
function weizPVP.Crosshair.SetLineThickness()
    if not XEN._display.UiMultiplier then
        return
    end
    CrosshairFrame.TopLine:SetIgnoreParentScale(true)
    CrosshairFrame.BottomLine:SetIgnoreParentScale(true)
    CrosshairFrame.LeftLine:SetIgnoreParentScale(true)
    CrosshairFrame.RightLine:SetIgnoreParentScale(true)
    CrosshairFrame.TopLine:SetWidth(weizPVP.Options.Crosshair.LineThickness * XEN._display.UiMultiplier)
    CrosshairFrame.BottomLine:SetWidth(weizPVP.Options.Crosshair.LineThickness * XEN._display.UiMultiplier)
    CrosshairFrame.LeftLine:SetHeight(weizPVP.Options.Crosshair.LineThickness * XEN._display.UiMultiplier)
    CrosshairFrame.RightLine:SetHeight(weizPVP.Options.Crosshair.LineThickness * XEN._display.UiMultiplier)
end

--> CHANGE SCALE
-------------------------------------------------------
function weizPVP.Crosshair.SetScale()
    CrosshairFrame:SetScale(weizPVP.Options.Crosshair.Scale)
end

-------------------------------------------------------------------------------
--|> FUNCTIONS: CONTROL
-------------------------------------------------------------------------------
--> RESET
-------------------------------------------------------
function weizPVP.Crosshair.Reset()
    HideCrosshair()
    targetNamePlateFrame = nil
    targetPlayerName = nil
    targetClassColor = nil
    targetClass = nil
end

--> ENABLE
-------------------------------------------------------
function weizPVP.Crosshair.Enable()
    if not weizPVP.Options.Crosshair.Enabled then
        return
    end
    weizPVP.Crosshair.Reset()
    --> Setup Core
    weizPVP.Crosshair.SetLineThickness()
    weizPVP.Crosshair.SetScale()
    weizPVP.Crosshair.SetAlpha()
    ConfigureRange()
    --> Net-o-Matic
    CrosshairFrame.NetOMatic:Show()
    --> Refresh crosshairs
    Crosshair.NewTarget()
    XEN.RescaleFrameAndChildren(CrosshairFrame)
end

--> DISABLE
function weizPVP.Crosshair.Disable()
    --> Clear nameplate and fade out
    targetNamePlateFrame = nil
    HideCrosshair()
end

--> TOGGLE
-------------------------------------------------------
function weizPVP.ToggleCrosshairs()
    if weizPVP.Options.Crosshair.Enabled == false then
        weizPVP.Options.Crosshair.Enabled = true
        weizPVP.Crosshair.Enable()
        weizPVP.PrintAddonMessage("Crosshairs are |cff37ff37ENABLED|r.")
    else
        weizPVP.Options.Crosshair.Enabled = false
        weizPVP.Crosshair.Disable()
        weizPVP.PrintAddonMessage("Crosshairs are |cffff3838DISABLED|r.")
    end
end

--> OnLoad
-------------------------------------------------------
function weizPVP.Crosshair.OnLoad()
    CrosshairFrame = weizPVP_CrosshairFrame
    HideCrosshair()
end

--> OnShow
-------------------------------------------------------
function weizPVP.Crosshair.OnShow()
    timer_ActiveTarget = C_Timer.NewTicker(0.3, TargetRangeCheck)
    ShowCrosshair()
end

--> OnHide
-------------------------------------------------------
function weizPVP.Crosshair.OnHide()
    if timer_ActiveTarget then
        timer_ActiveTarget:Cancel()
    end
    HideCrosshair()
end
