-------------------------------------------------------------------------------
--|> CONFIG
--: Slash Commands and Interface Options
-------------------------------------------------------------------------------
--|> Upvalue Globals
-------------------------------------------------------------------------------
local weizPVP = weizPVP
local GetCVarDefault = GetCVarDefault
local InterfaceOptionsFrameAddOnsListScrollBar = InterfaceOptionsFrameAddOnsListScrollBar
local InterfaceOptionsFrame_OpenToCategory = InterfaceOptionsFrame_OpenToCategory
local wipe = wipe
local _

-------------------------------------------------------------------------------
--|> LIBS
-------------------------------------------------------------------------------
local SM = LibStub("LibSharedMedia-3.0")
local sounds = SM:List("sound")

-------------------------------------------------------------------------------
--|> CROSSHAIR OPTIONS PANEL
-------------------------------------------------------------------------------
local crosshairPanel = CreateFrame("Frame", "weizPVP_CrosshairOptionsPanel", InterfaceOptionsFramePanelContainer)
crosshairPanel:Hide()

-------------------------------------------------------------------------------
--|> SLASH COMMANDS
-------------------------------------------------------------------------------
local Smax
--=> Commands Option Table
-------------------------------------------------------------------------------
weizPVP.SlashCommands = {
    name = "Slash Commands",
    order = -3,
    type = "group",
    args = {
        intro = {
            name = "weizPVP Slash Commands",
            type = "description",
            order = 1,
            cmdHidden = true
        },
        config = {
            name = "config",
            desc = "Opens the configuration menu.",
            type = "execute",
            func = function()
                InterfaceOptionsFrame_OpenToCategory("weizPVP")
                _, Smax = InterfaceOptionsFrameAddOnsListScrollBar:GetMinMaxValues()
                InterfaceOptionsFrameAddOnsListScrollBar:SetValue(Smax)
                InterfaceOptionsFrame_OpenToCategory("weizPVP")
            end,
            guiHidden = true,
            order = 2
        },
        show = {
            name = "Show",
            desc = "Show the weizPVP window.",
            type = "execute",
            order = 3,
            func = function()
                weizPVP.SetWindowVisible(true)
                weizPVP.Options.Window.Visible = true
            end,
            dialogHidden = true
        },
        hide = {
            name = "Hide",
            desc = "Hide the weizPVP window.",
            type = "execute",
            order = 4,
            func = function()
                weizPVP.SetWindowVisible(false)
                weizPVP.Options.Window.Visible = false
            end,
            dialogHidden = true
        },
        lock = {
            name = "Lock",
            desc = "Toggle locking of the window's position and size.",
            type = "execute",
            order = 5,
            func = function()
                weizPVP.SetWindowLocked(not weizPVP.Options.Window.Locked)
            end,
            dialogHidden = true
        },
        pin = {
            name = "Pin",
            desc = "Toggle pinning of the window.",
            type = "execute",
            order = 6,
            func = function()
                if weizPVP.Options.Window.Locked then
                    weizPVP.PrintAddonMessage("Window is currently |cffff3838LOCKED|r. Window can only be pinned after being unlocked.")
                else
                    weizPVP.SetWindowPin(not weizPVP.Options.Window.Pinned)
                end
            end,
            dialogHidden = true
        },
        ch = {
            name = "Crosshairs",
            desc = "Toggles the Crosshairs on or off",
            type = "execute",
            order = 7,
            func = function()
                if weizPVP.Options.Crosshair.Enabled == false then
                    weizPVP.Options.Crosshair.Enabled = true
                    weizPVP.Crosshair.Enable()
                    weizPVP.PrintAddonMessage("Crosshairs are |cff37ff37ENABLED|r.")
                else
                    weizPVP.Options.Crosshair.Enabled = false
                    weizPVP.Crosshair.Disable()
                    weizPVP.PrintAddonMessage("Crosshairs are |cffff3838DISABLED|r.")
                end
            end,
            dialogHidden = true
        },
        pb = {
            name = "Player Browser",
            desc = "Toggle the Player Browser",
            type = "execute",
            order = 8,
            func = function()
                if weizPVP.PlayerBrowser:IsShown() then
                    weizPVP.PlayerBrowser:Hide()
                else
                    weizPVP.PlayerBrowser:Show()
                end
            end,
            dialogHidden = true
        },
        --> DEBUG: Detects and manages ALL player characters; not just enemies.
        debug = {
            name = "DEBUG",
            desc = "Toggles DEBUG mode where ALL players of any factions are accepted as valid.",
            type = "execute",
            order = 60,
            func = function()
                if weizPVP.Options.DEBUG == true then
                    weizPVP.Options.DEBUG = false
                    weizPVP.ClearListData()
                    weizPVP.PrintAddonMessage("|TInterface\\Addons\\weizPVP\\Media\\Icons\\Pack\\toolbox.tga:0|t - |cff37ff37DISABLED|r")
                    weizPVP.SetStatusBarMessage("|cff37ff37> DEBUG MODE DISABLED <|r", true, 3)
                else
                    weizPVP.Options.DEBUG = true
                    weizPVP.PrintAddonMessage("|TInterface\\Addons\\weizPVP\\Media\\Icons\\Pack\\toolbox.tga:0|t - |cffff3838ENABLED|r")
                    weizPVP.SetStatusBarMessage("|cffff3838>|r DEBUG MODE: |cffff3838ACTIVE <|r", true, 3)
                end
            end,
            cmdHidden = true
        },
        --> RESETALL: Wipes EVERYTHING
        resetall = {
            name = "Reset All",
            desc = "Resets all settings and wipes player data",
            type = "execute",
            order = 61,
            func = function()
                weizPVP.ResetAll()
            end,
            cmdHidden = true
        },
        --> RESET OPTIONS
        resetoptions = {
            name = "Reset Options",
            desc = "Resets all Options but doesn't change player data at all",
            type = "execute",
            order = 63,
            func = function()
                weizPVP.ResetOptions()
            end,
            cmdHidden = true
        },
        --> RESET PLAYER DB
        resetplayers = {
            name = "Reset Player DB",
            desc = "Wipes the player database. Options remain untouched.",
            type = "execute",
            order = 64,
            func = function()
                weizPVP.ClearListData()
                if weizPVP.PlayerDB then
                    wipe(weizPVP.PlayerDB)
                end
                weizPVP.globalDB.PlayerDB = {}
            end,
            cmdHidden = true
        }
        -- !! END HIDDEN COMMANDS =====================================================
    }
}

--=> Register Slash Commands
-------------------------------------------------------------------------------
LibStub("AceConfig-3.0"):RegisterOptionsTable("weizPVP.SlashCommands", weizPVP.SlashCommands, {"wpvp", "weizpvp"})
-------------------------------------------------------------------------------
--|> LOCAL FUNCTIONS
-------------------------------------------------------------------------------
local ValidateNumeric = function(_, val)
    if val ~= nil and val ~= "" and not tonumber(val) then
        return false
    end
    return true
end
-------------------------------------------------------------------------------
--|> INTERFACE OPTIONS: WOW GUI MENU
-------------------------------------------------------------------------------
weizPVP.OptionsTable = {
    name = "weizPVP  |cffffffff/|r  v" .. weizPVP.Addon.VERSION,
    type = "group",
    args = {
        --> General Options Group
        -------------------------------------------
        general = {
            name = "General",
            type = "group",
            order = 1,
            args = {
                introGeneral = {
                    name = " |cffffa012GENERAL|r",
                    type = "header",
                    order = 1,
                    width = "full"
                },
                --> General Options
                -------------------------------------------
                minimapIcon = {
                    name = "Show minimap icon",
                    desc = "Show or hide the minimap icon.",
                    type = "toggle",
                    width = "full",
                    order = 2,
                    get = function()
                        return weizPVP.Options.LDB.minimap
                    end,
                    set = function(_, value)
                        if value == true then
                            LibStub("LibDBIcon-1.0"):Show("weizPVP")
                        else
                            LibStub("LibDBIcon-1.0"):Hide("weizPVP")
                        end
                        weizPVP.Options.LDB.minimap = value
                    end
                },
                EnableBGs = {
                    name = " Enabled in BGs",
                    desc = "Enable weizPVP in Battlegrounds.",
                    type = "toggle",
                    order = 3,
                    width = "full",
                    get = function()
                        return weizPVP.Options.Addon.EnabledInBattlegrounds
                    end,
                    set = function(_, value)
                        --> Double check that we were already disabled. If so, initialize addon again.
                        weizPVP.Options.Addon.EnabledInBattlegrounds = value
                        weizPVP.GetPVPZone()
                    end
                },
                EnableArenas = {
                    name = " Enabled in Arenas",
                    desc = "Enable weizPVP in Arenas.",
                    type = "toggle",
                    order = 4,
                    width = "full",
                    get = function()
                        return weizPVP.Options.Addon.EnabledInArena
                    end,
                    set = function(_, value)
                        --> Double check that we were already disabled. If so, initialize addon again.
                        weizPVP.Options.Addon.EnabledInArena = value
                        weizPVP.GetPVPZone()
                    end
                }
            }
        },
        --> Alerts Group
        -------------------------------------------
        alerts = {
            name = "Alerts",
            type = "group",
            order = 4,
            args = {
                introAlerts = {
                    name = " |cffffa012ALERTS & NOTIFICATIONS|r",
                    type = "header",
                    order = 1,
                    width = "full"
                },
                --> New player detected
                -------------------------------------------
                AlertDetectedSection = {
                    name = "\n |cffffa012On New Player Detection|r",
                    type = "description",
                    fontSize = "large",
                    width = "full",
                    order = 2
                },
                AlertDetectedPlayer = {
                    name = " New Player Detected: Audio Alert",
                    desc = "Play's an alert when a new player is added to the enemy list." ..
                        "Plays the audio until it ends; doesnt not overlap with multiple detections.",
                    type = "toggle",
                    width = "full",
                    order = 3,
                    get = function()
                        return weizPVP.Options.AudioAlerts.DetectedPlayerSound
                    end,
                    set = function(_, value)
                        weizPVP.Options.AudioAlerts.DetectedPlayerSound = value
                    end
                },
                AlertDetectedPlayerSoundFile = {
                    type = "select",
                    name = " New Player Alert Sound",
                    desc = "The audio file that plays on detection of a new player.",
                    values = sounds,
                    width = "full",
                    order = 4,
                    get = function()
                        for i, v in next, sounds do
                            if v == weizPVP.Options.AudioAlerts.DetectedPlayerSoundFile then
                                return i
                            end
                        end
                    end,
                    set = function(_, value)
                        weizPVP.Options.AudioAlerts.DetectedPlayerSoundFile = sounds[value]
                    end,
                    itemControl = "DDI-Sound",
                    disabled = function()
                        return not weizPVP.Options.AudioAlerts.DetectedPlayerSound
                    end
                },
                --> Stealth
                stealthSection = {
                    name = "\n |cffffa012Stealth|r",
                    type = "description",
                    fontSize = "large",
                    width = "full",
                    order = 5
                },
                StealthAlertEnabled = {
                    name = " Enable Stealth Alerts",
                    desc = "Show an alert when a unit is detected going stealth or invis.",
                    type = "toggle",
                    width = "full",
                    order = 6,
                    get = function()
                        return weizPVP.Options.StealthAlert.Enabled
                    end,
                    set = function(_, value)
                        weizPVP.Options.StealthAlert.Enabled = value
                    end
                },
                StealthAlertSoundEnabled = {
                    name = " Enable Stealth Audio Alert",
                    desc = "Plays a sound when players going stealth or invis while Stealth Alerts is enabled.",
                    type = "toggle",
                    width = "full",
                    order = 7,
                    get = function()
                        return weizPVP.Options.StealthAlert.EnableSound
                    end,
                    set = function(_, value)
                        weizPVP.Options.StealthAlert.EnableSound = value
                    end,
                    disabled = function()
                        return not weizPVP.Options.StealthAlert.Enabled
                    end
                },
                StealthAlertSoundFile = {
                    type = "select",
                    name = " Stealth Alert Sound",
                    desc = "The sound to play on stealth detection, if enabled.",
                    values = sounds,
                    width = "full",
                    order = 8,
                    get = function()
                        for i, v in next, sounds do
                            if v == weizPVP.Options.StealthAlert.SoundFile then
                                return i
                            end
                        end
                    end,
                    set = function(_, value)
                        weizPVP.Options.StealthAlert.SoundFile = sounds[value]
                    end,
                    itemControl = "DDI-Sound",
                    disabled = function()
                        if weizPVP.Options.StealthAlert.EnableSound == false or weizPVP.Options.StealthAlert.Enabled == false then
                            return true
                        else
                            return false
                        end
                    end
                },
                --> Phasing
                phasingSection = {
                    name = "\n |cffffa012Phasing|r",
                    type = "description",
                    fontSize = "large",
                    width = "full",
                    order = 9
                },
                PhasingChatAlertEnabled = {
                    name = " Show phasing alert in chat",
                    desc = "Puts a notice in chat when your character is phasing.",
                    type = "toggle",
                    width = "full",
                    order = 10,
                    get = function()
                        return weizPVP.Options.Alerts.PhasingChat
                    end,
                    set = function(_, value)
                        weizPVP.Options.Alerts.PhasingChat = value
                    end
                }
            }
        },
        --> KOS Group
        -------------------------------------------
        kos = {
            name = "Kill On Sight",
            type = "group",
            order = 5,
            args = {
                introKOS = {
                    name = " |cffffa012Kill On Sight (KOS)|r",
                    type = "header",
                    order = 1,
                    width = "full"
                },
                --> New player detected
                -------------------------------------------
                AudioAlert = {
                    name = " Audio Alert on detect",
                    desc = "Play's an alert when a player on the KOS list has been found.",
                    type = "toggle",
                    width = "full",
                    order = 2,
                    get = function()
                        return weizPVP.Options.KOS.AudioAlert
                    end,
                    set = function(_, value)
                        weizPVP.Options.KOS.AudioAlert = value
                    end
                },
                AudioAlertFile = {
                    type = "select",
                    name = " KOS Player Sound",
                    desc = "The audio file that plays on detection of a new player.",
                    values = sounds,
                    width = "full",
                    order = 3,
                    get = function()
                        for i, v in next, sounds do
                            if v == weizPVP.Options.KOS.AudioAlertFile then
                                return i
                            end
                        end
                    end,
                    set = function(_, value)
                        weizPVP.Options.KOS.AudioAlertFile = sounds[value]
                    end,
                    itemControl = "DDI-Sound",
                    disabled = function()
                        return not weizPVP.Options.KOS.AudioAlert
                    end
                },
                ChatOutput = {
                    type = "toggle",
                    name = " Chat alert",
                    desc = "Outputs an alert for the detected player in chat (not sent to any chat channel)",
                    width = "full",
                    order = 5,
                    get = function()
                        return weizPVP.Options.KOS.ChatAlert
                    end,
                    set = function(_, value)
                        weizPVP.Options.KOS.ChatAlert = value
                    end
                },
                TaskbarAlert = {
                    type = "toggle",
                    name = " Flash the taskbar icon on KOS detect",
                    desc = "Flashes the WoW application icon on your taskbar/dock when a KOS target is detected. Useful for when WoW is minimized, using multiple monitors, etc.",
                    width = "full",
                    order = 6,
                    get = function()
                        return weizPVP.Options.KOS.TaskbarAlert
                    end,
                    set = function(_, value)
                        weizPVP.Options.KOS.TaskbarAlert = value
                    end
                }
            }
        },
        --> Crosshair Group
        -------------------------------------------
        crosshair = {
            name = "Crosshair",
            type = "group",
            order = 6,
            args = {
                intro = {
                    name = "|cffffa012CROSSHAIR|r",
                    type = "header",
                    order = 1,
                    width = "full"
                },
                --> Crosshair Options
                -------------------------------------------
                Enabled = {
                    name = " Enable Crosshairs",
                    desc = "Toggles the Crosshairs on/off",
                    type = "toggle",
                    width = "full",
                    order = 2,
                    get = function()
                        return weizPVP.Options.Crosshair.Enabled
                    end,
                    set = function(_, value)
                        if value == true then
                            weizPVP.Options.Crosshair.Enabled = true
                            weizPVP.Crosshair.Enable()
                        else
                            weizPVP.Options.Crosshair.Enabled = false
                            weizPVP.Crosshair.Disable()
                        end
                    end
                },
                --> RANGE
                -------------------------------------------
                ShowRange = {
                    name = " Show Range Text",
                    desc = "Show the estimated range of the target while the crosshair is active. Text fades away under 30 yards.",
                    type = "toggle",
                    width = "full",
                    order = 10,
                    get = function()
                        return weizPVP.Options.Crosshair.ShowRange
                    end,
                    set = function(_, value)
                        if value == true then
                            weizPVP.Options.Crosshair.ShowRange = true
                            _G.weizPVP_CrosshairFrame.RangeText:Show()
                            weizPVP.Crosshair.Enable()
                        else
                            weizPVP.Options.Crosshair.ShowRange = false
                            _G.weizPVP_CrosshairFrame.RangeText:Hide()
                            weizPVP.Crosshair.Enable()
                        end
                    end,
                    disabled = function()
                        return not weizPVP.Options.Crosshair.Enabled
                    end
                },
                --> GUILD AND TEXT
                -------------------------------------------
                NameEnabled = {
                    name = " Enable Name Text",
                    desc = "Toggles the the target's name being shown on the Crosshair.",
                    type = "toggle",
                    width = "full",
                    order = 11,
                    get = function()
                        return weizPVP.Options.Crosshair.NameEnabled
                    end,
                    set = function(_, value)
                        weizPVP.Options.Crosshair.NameEnabled = value
                        weizPVP.Crosshair.Reset()
                        weizPVP.Crosshair.NewTarget()
                    end,
                    disabled = function()
                        return not weizPVP.Options.Crosshair.Enabled
                    end
                },
                GuildEnabled = {
                    name = " Enable Guild Text",
                    desc = "Toggles the the target's guild being shown on the Crosshair.",
                    type = "toggle",
                    width = "full",
                    order = 12,
                    get = function()
                        return weizPVP.Options.Crosshair.GuildEnabled
                    end,
                    set = function(_, value)
                        weizPVP.Options.Crosshair.GuildEnabled = value
                        weizPVP.Crosshair.Reset()
                        weizPVP.Crosshair.NewTarget()
                    end,
                    disabled = function()
                        return not weizPVP.Options.Crosshair.Enabled
                    end
                },
                -->> SLIDERS  <<--
                Alpha = {
                    name = " Alpha Multiplier",
                    desc = "Adjust Crosshair Alpha. 0 = transparent, 1.0 = max opacity/visability",
                    type = "range",
                    min = 0,
                    max = 1,
                    width = "full",
                    validate = ValidateNumeric,
                    step = 0.05,
                    order = 20,
                    get = function()
                        if type(weizPVP.Options.Crosshair.Alpha) == "number" then
                            return weizPVP.Options.Crosshair.Alpha
                        else
                            return 1
                        end
                    end,
                    set = function(_, value)
                        if type(weizPVP.Options.Crosshair.Alpha) == "number" then
                            weizPVP.Options.Crosshair.Alpha = value
                            weizPVP.Crosshair.SetAlpha()
                        else
                            weizPVP.Options.Crosshair.Alpha = 1
                            weizPVP.Crosshair.SetAlpha()
                        end
                    end,
                    disabled = function()
                        return not weizPVP.Options.Crosshair.Enabled
                    end
                },
                -->> SCALES
                Scale = {
                    name = " Scale",
                    desc = "Adjust the Scale of the Crosshair",
                    type = "range",
                    min = 0.1,
                    max = 4,
                    width = "full",
                    validate = ValidateNumeric,
                    step = 0.1,
                    order = 21,
                    get = function()
                        if type(weizPVP.Options.Crosshair.Scale) == "number" then
                            return weizPVP.Options.Crosshair.Scale
                        else
                            return 1
                        end
                    end,
                    set = function(_, value)
                        if type(weizPVP.Options.Crosshair.Scale) == "number" then
                            weizPVP.Options.Crosshair.Scale = value
                            weizPVP.Crosshair.SetScale(value)
                        else
                            weizPVP.Options.Crosshair.Scale = 1
                            weizPVP.Crosshair.SetScale(1)
                        end
                    end,
                    disabled = function()
                        return not weizPVP.Options.Crosshair.Enabled
                    end
                },
                -->> LINE THICKNESS
                LineThickness = {
                    name = " Line Thickness",
                    desc = "Change the thickness of the lines",
                    type = "range",
                    min = 1,
                    max = 12,
                    width = "full",
                    step = 1,
                    order = 22,
                    get = function()
                        return weizPVP.Options.Crosshair.LineThickness
                    end,
                    set = function(_, value)
                        weizPVP.Options.Crosshair.LineThickness = value
                        weizPVP.Crosshair.SetLineThickness(value)
                    end,
                    disabled = function()
                        return not weizPVP.Options.Crosshair.Enabled
                    end
                },
                --> Crosshair Tips
                -------------------------------------------
                CVarHeader = {
                    name = "\n|cffffa012Nameplate CVars:|r",
                    type = "description",
                    fontSize = "large",
                    width = "full",
                    order = 25
                },
                nameplateTargetRadialPosition = {
                    type = "toggle",
                    name = "|cff42dcf4nameplateTargetRadialPosition|r (|cff37ff37Enabled is recommended|r)",
                    desc = "When target is off screen, position its nameplate radially around sides and bottom",
                    width = "full",
                    order = 27,
                    get = function()
                        return GetCVar("nameplateTargetRadialPosition") == "1" and true or false
                    end,
                    set = function(_, value)
                        if value then
                            SetCVar("nameplateTargetRadialPosition", "1")
                        else
                            SetCVar("nameplateTargetRadialPosition", "0")
                        end
                    end
                },
                nameplateTargetBehindMaxDistance = {
                    type = "range",
                    name = "|cff42dcf4nameplateTargetBehindMaxDistance|r |cffffffff(|r|cff37ff3760 is recommended|r|cffffffff)|r",
                    desc = "The max distance to show the target nameplate when the target is behind the camera.",
                    width = "full",
                    min = 15,
                    max = 60,
                    validate = ValidateNumeric,
                    step = 1,
                    order = 28,
                    get = function()
                        return tonumber(GetCVar("nameplateTargetBehindMaxDistance"))
                    end,
                    set = function(_, value)
                        SetCVar("nameplateTargetBehindMaxDistance", tostring(value))
                    end
                },
                ResetCVars = {
                    type = "execute",
                    name = "Reset the above CVars to default values",
                    desc = "The CVars nameplateTargetRadialPosition, nameplateTargetBehindMaxDistance, and nameplateMaxDistance will be reset to default",
                    width = "full",
                    order = 40,
                    func = function()
                        SetCVar("nameplateTargetRadialPosition", GetCVarDefault("nameplateTargetRadialPosition"))
                        SetCVar("nameplateTargetBehindMaxDistance", GetCVarDefault("nameplateTargetBehindMaxDistance"))
                        weizPVP.PrintAddonMessage("|cff27e817CVars reset|r :")
                    end
                }
            }
        }
    }
}

LibStub("AceConfig-3.0"):RegisterOptionsTable("weizPVP.OptionsTable", weizPVP.OptionsTable)
LibStub("AceConfigDialog-3.0"):AddToBlizOptions("weizPVP.OptionsTable", "weizPVP")
