-------------------------------------------------------------------------------
--|> DATA PROCESSING
--: Functions that help acquire and manage data involving the PlayerDB
-------------------------------------------------------------------------------
--|> Upvalue Globals
-------------------------------------------------------------------------------
local weizPVP = weizPVP
local UnitName = UnitName
local GetGuildInfo = GetGuildInfo
local GetTime = GetTime
local _, NS = ...

--> UpdatePlayerActiveCache
--:  Updates data in the PlayerActiveCache and then calls
--: Sends data off the Lists to be processed
-----------------------------------------------------------------------
function weizPVP.UpdatePlayerActiveCache(name, health, stealth, dead, role, GUID)
    --> Verify GUIDexists
    if not GUID then
        return
    end
    local newPlayerOnList
    --: Check for player already in cache
    if weizPVP.PlayerActiveCache[GUID] then
        newPlayerOnList = false
    else
        weizPVP.PlayerActiveCache[GUID] = {}
        newPlayerOnList = true
    end
    --: Update PlayerCache info
    if newPlayerOnList then
        weizPVP.PlayerActiveCache[GUID].GUID = GUID
        --> NAME
        if name then
            weizPVP.PlayerActiveCache[GUID].Name = name
        elseif weizPVP.PlayerDB[name].Name then
            weizPVP.PlayerActiveCache[GUID].Name = weizPVP.PlayerDB[name].Name
        end
        --> LEVEL
        if weizPVP.PlayerDB[name].Level then
            weizPVP.PlayerActiveCache[GUID].Level = weizPVP.PlayerDB[name].Level
            weizPVP.PlayerActiveCache[GUID].Estimated = weizPVP.PlayerDB[name].Estimated
        else -- no level found
            weizPVP.PlayerActiveCache[GUID].Level = 0
            weizPVP.PlayerActiveCache[GUID].Estimated = true
            weizPVP.PlayerDB[name].Level = 0
            weizPVP.PlayerDB[name].Estimated = true
        end
        --> CLASS
        if weizPVP.PlayerDB[name].Class then
            weizPVP.PlayerActiveCache[GUID].Class = weizPVP.PlayerDB[name].Class
        end
        --> GUILD
        if weizPVP.PlayerDB[name].Guild then
            weizPVP.PlayerActiveCache[GUID].Guild = weizPVP.PlayerDB[name].Guild
        end
        --> HEALTH
        if health ~= nil then
            weizPVP.PlayerActiveCache[GUID].Health = health
        elseif weizPVP.PlayerActiveCache[GUID].Health == nil then
            weizPVP.PlayerActiveCache[GUID].Health = 100
        end
        --> STEALTH
        if stealth ~= nil then
            weizPVP.PlayerActiveCache[GUID].Stealth = stealth
        end
        --> DEAD
        if dead ~= nil then
            weizPVP.PlayerActiveCache[GUID].Dead = dead
            if dead == true then
                weizPVP.PlayerActiveCache[GUID].Health = 0
            end
        end
        --> ROLE
        if role ~= nil then
            weizPVP.PlayerActiveCache[GUID].Role = role
            weizPVP.PlayerDB[name].Role = role
        end
    end
    --> Add player Data to lists
    weizPVP.AddPlayerDataToNearby(GUID, newPlayerOnList)
end

--> Static Role Assignment
local function RoleAssign(class)
    if class == "ROGUE" or class == "MAGE" or class == "WARLOCK" or class == "HUNTER" then
        return "DAMAGER"
    end
    return nil
end

--> GET UNIT DATA
-- Gets data for events with a UNITID, such as 'target' or 'nameplate'
local UnitDataThrottle = 2
function weizPVP.GetUnitData(unit)
    if not UnitExists(unit) then -- unit check
        return
    end
    if weizPVP.IsUnitValidForTracking(unit) then -- Valid player?
        local timestamp = GetTime()
        local name = weizPVP.GetUnitName(unit)
        local GUID = UnitGUID(unit) or nil
        -- Unit has valid name and GUID
        if name and GUID then
            local guild = GetGuildInfo(unit)
            local class
            _, class = UnitClass(unit)
            local role = RoleAssign(UnitClass(unit))
            local level = UnitLevel(unit)
            local race = UnitRace(unit)
            local Dead = UnitIsDeadOrGhost(unit) or nil
            local Health = (UnitHealth(unit) / UnitHealthMax(unit) * 100) or nil
            local OnTaxi = UnitOnTaxi(unit)
            -- New DB Player?
            if not weizPVP.PlayerDB[name] then
                weizPVP.PlayerDB[name] = {}
                weizPVP.PlayerDB[name].Estimated = false
                weizPVP.PlayerDB[name].Name = name
                weizPVP.PlayerDB[name].Guild = guild
                weizPVP.PlayerDB[name].Class = class
                if role ~= nil then
                    weizPVP.PlayerDB[name].Role = role
                end
                weizPVP.PlayerDB[name].Level = level
                weizPVP.PlayerDB[name].Race = race
                weizPVP.PlayerDB[name].Timestamp = timestamp
            end
            -- New Cache Player
            if weizPVP.PlayerDB[name].Estimated == true or (weizPVP.PlayerDB[name].Timestamp + UnitDataThrottle < timestamp) then
                if not weizPVP.NearbyList[name] then -- Check if player is new to cache/list
                    -- Player new to cache; get data
                    weizPVP.PlayerDB[name].Estimated = false
                    weizPVP.PlayerDB[name].Guild = guild
                    weizPVP.PlayerDB[name].Race = race
                    weizPVP.PlayerDB[name].Level = level
                    if role ~= nil then
                        weizPVP.PlayerDB[name].Role = role
                    end
                    weizPVP.PlayerDB[name].Timestamp = timestamp
                    -- Active visible player
                    if weizPVP.PlayersOnBars[GUID] and weizPVP.PlayerActiveCache[GUID] then
                        if OnTaxi and not weizPVP.PlayerActiveCache[GUID].OnTaxi then
                            weizPVP.PlayerActiveCache[GUID].OnTaxi = true
                        elseif not OnTaxi and weizPVP.PlayerActiveCache[GUID].OnTaxi == true then
                            weizPVP.PlayerActiveCache[GUID].OnTaxi = false
                        end
                        if UnitCanAttack("player", unit) then
                            weizPVP.PlayerActiveCache[GUID].Unattackable = nil
                        else
                            weizPVP.PlayerActiveCache[GUID].Unattackable = true
                        end
                    end
                    -- Update player data on bars
                    weizPVP.UpdatePlayerActiveCache(name, Health, false, Dead, role, GUID)
                    return
                end
            end
            weizPVP.PlayerDB[name].Estimated = false
            weizPVP.PlayerDB[name].Timestamp = timestamp
            if weizPVP.PlayersOnBars[GUID] and weizPVP.PlayerActiveCache[GUID] then
                if OnTaxi and not weizPVP.PlayerActiveCache[GUID].OnTaxi == true then
                    weizPVP.PlayerActiveCache[GUID].OnTaxi = true
                elseif not OnTaxi and weizPVP.PlayerActiveCache[GUID].OnTaxi == true then
                    weizPVP.PlayerActiveCache[GUID].OnTaxi = false
                end
            end
            weizPVP.UpdatePlayerActiveCache(name, Health, nil, Dead, role, GUID)
            return
        end
    end
end

--> Is Unit Valid For Tracking
function weizPVP.IsUnitValidForTracking(unit)
    if (UnitCanAttack("player", unit) or weizPVP.Options.DEBUG) and UnitIsPlayer(unit) then
        if not string.find(UnitGUID(unit), "Player") then
            return false
        end
        return true
    end
    return false
end

--> weizPVP's GetUnitName()
--: Returns "playerName-realmName" for all players
local gsub = gsub
function weizPVP.GetUnitName(unit)
    if not unit then -- check for unit
        return
    end
    local name, realm = UnitName(unit, true) -- check for unit name
    if not name then
        return nil
    end
    if not realm then -- same realm
        return name .. "-" .. NS.PlayerRealm -- name + player's realm
    else -- different server
        realm = gsub(realm, "[%s%-]", "") --strip
        return name .. "-" .. realm
    end
end
