-------------------------------------------------------------------------------------------
--|> EVENTS
-------------------------------------------------------------------------------------------
--|> Upvalue Globals
-------------------------------------------------------------------------------------------
local _, NS = ...
local weizPVP = weizPVP
local GetClassColor = GetClassColor
local WrapTextInColorCode = WrapTextInColorCode
local UnitIsDeadOrGhost = UnitIsDeadOrGhost
local GetTime = GetTime
local IsInInstance = IsInInstance
local bit = bit
local select = select
local bit_band = bit.band
local C_Timer_After = C_Timer.After
local InCombatLockdown = InCombatLockdown
local CombatLogGetCurrentEventInfo = CombatLogGetCurrentEventInfo
local strsub = strsub
local wipe = wipe
local UnitExists = UnitExists
local GetPlayerInfoByGUID = GetPlayerInfoByGUID
local GetSpecializationInfoByID = GetSpecializationInfoByID
local GetZoneText = GetZoneText
local GetFramerate = GetFramerate
local GarrisonLandingPageMinimapButton = GarrisonLandingPageMinimapButton

--|> INIT GLOBALS
-------------------------------------------------------------------------------------------
weizPVP.PlayerActiveCache = {}

--|> EVENT MONITORING: ENABLE / DISABLE
-------------------------------------------------------------------------------------------
--> ENABLE EVENTS
function weizPVP.EnableEvents()
    --> DATA COLLECTION
    weizPVP:RegisterEvent("NAME_PLATE_UNIT_ADDED", "NameplateAdded")
    weizPVP:RegisterEvent("NAME_PLATE_UNIT_REMOVED", "NameplateRemoved")
    weizPVP:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "CombatLogEvent")
    weizPVP:RegisterEvent("PLAYER_TARGET_CHANGED", "PlayerTargetEvent")
    weizPVP:RegisterEvent("UPDATE_MOUSEOVER_UNIT", "PlayerMouseoverEvent")
    weizPVP:RegisterEvent("UNIT_HEALTH_FREQUENT", "UnitHealthEvent")
    weizPVP:RegisterEvent("UNIT_TARGET", "UnitTargetEvent")

    --> ZONE CHANGED
    weizPVP:RegisterEvent("PLAYER_ENTERING_BATTLEGROUND", "PlayerEnteringBattlegroundEvent")
    weizPVP:RegisterEvent("PLAYER_ENTERING_WORLD", "PlayerEnteringWorldEvent")
    weizPVP:RegisterEvent("ZONE_CHANGED_NEW_AREA", "ZoneChangedNewAreaEvent")

    --> COMBAT/PVP
    weizPVP:RegisterEvent("PLAYER_REGEN_DISABLED", "EnteringCombat")
    weizPVP:RegisterEvent("PLAYER_REGEN_ENABLED", "LeavingCombat")

    --> PLAYER UPDATES
    weizPVP:RegisterEvent("PLAYER_LEVEL_UP", "PlayerLevelUp")

    --> DISPLAY CHANGES
    weizPVP:RegisterEvent("DISPLAY_SIZE_CHANGED", "UpdateDisplayData")
    weizPVP:RegisterEvent("PLAYER_LOGIN", "UpdateDisplayData")

    --> Crosshairs
    weizPVP.Crosshair.Enable()
end

--> DISABLE EVENTS
-----------------------------------------------------------------------
function weizPVP.DisableEvents()
    --> DATA COLLECTION
    weizPVP:UnregisterEvent("NAME_PLATE_UNIT_ADDED", "NameplateAdded")
    weizPVP:UnregisterEvent("NAME_PLATE_UNIT_REMOVED", "NameplateRemoved")
    weizPVP:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "CombatLogEvent")
    weizPVP:UnregisterEvent("PLAYER_TARGET_CHANGED", "PlayerTargetEvent")
    weizPVP:UnregisterEvent("UPDATE_MOUSEOVER_UNIT", "PlayerMouseoverEvent")
    weizPVP:UnregisterEvent("UNIT_HEALTH_FREQUENT", "UnitHealthEvent")
    weizPVP:UnregisterEvent("UNIT_TARGET", "UnitTargetEvent")

    --> COMBAT/PVP
    weizPVP:UnregisterEvent("PLAYER_REGEN_DISABLED", "EnteringCombat")
    weizPVP:UnregisterEvent("PLAYER_REGEN_ENABLED", "LeavingCombat")

    --> PLAYER UPDATES
    weizPVP:UnregisterEvent("PLAYER_LEVEL_UP", "PlayerLevelUp")

    --> DISPLAY CHANGES
    weizPVP:UnregisterEvent("DISPLAY_SIZE_CHANGED", "UpdateDisplayData")
    weizPVP:UnregisterEvent("PLAYER_LOGIN", "UpdateDisplayData")

    -->Crosshairs
    weizPVP.Crosshair.Disable()
end

--|> ZONE INFO
-------------------------------------------------------------------------------------------
local inInstance
local ZoneTimerStartTime = GetTime()
local ValidBG
local ValidWorldZone

--> Check for Valid (PVP) Instance
local function CheckValidInstance()
    --> Checking for Valid BG
    inInstance = select(2, IsInInstance()) --> Get instance info
    if inInstance == "pvp" and weizPVP.Options.Addon.EnabledInBattlegrounds == true then
        return true
    end
    if inInstance == "arena" and weizPVP.Options.Addon.EnabledInArena == true then
        return true
    end
    return false
end

--> Check for Valid World Zone
local function CheckValidWorldZone()
    --> Checking for Valid WPVP Zone
    inInstance = select(2, IsInInstance())
    if inInstance == "none" then
        return true
    else
        return false
    end
end

--> Get PVP Zone
function weizPVP.GetPVPZone()
    if ZoneTimerStartTime + 1 <= GetTime() then
        ZoneTimerStartTime = GetTime()
        if GetZoneText() == "" then
            C_Timer_After(0.5, weizPVP.GetPVPZone)
            return
        end
        ValidWorldZone = CheckValidWorldZone()
        ValidBG = CheckValidInstance()
        if not InCombatLockdown() then
            if ValidWorldZone or ValidBG then
                weizPVP:OnEnable()
                weizPVP.HeaderFrame:Show()
                if weizPVP.Options.Window.Collapsed == false then
                    weizPVP.CoreFrame:Show()
                end
            else
                weizPVP:OnDisable()
            end
        end
    else
        return
    end
end

--> EVENT: Entering Battleground Instance
function weizPVP:PlayerEnteringBattlegroundEvent()
    C_Timer_After(1, self.GetPVPZone)
    wipe(weizPVP.CurrentNameplates)
end

--> EVENT: ZoneChangedNewAreaEvent
function weizPVP:ZoneChangedNewAreaEvent()
    C_Timer_After(1, self.GetPVPZone)
end

--> EVENT: Player Entering World
function weizPVP:PlayerEnteringWorldEvent()
    wipe(weizPVP.CurrentNameplates)
    C_Timer_After(1, self.GetPVPZone)
end

--|> COMBAT LOG FUNCTIONS
-------------------------------------------------------------------------------------------
--> Static Role Assignment
local function RoleAssign(class)
    if class == "ROGUE" or class == "MAGE" or class == "WARLOCK" or class == "HUNTER" then
        return "DAMAGER"
    end
    return nil
end

--> Get Role from Spec
local function GetSpecRole(specid)
    if specid then
        return select(5, GetSpecializationInfoByID(specid))
    end
    return nil
end

--> Role Check
local function ClRoleCheck(spellID, class)
    local role = RoleAssign(class)
    if role ~= nil then
        return role
    end

    --> Check Spell ID against DB of specs/roles
    local specIdCheck = weizPVP.GetSpecIdFromSpellId(spellID)
    if not specIdCheck then
        return nil
    end

    --> Role
    role = GetSpecRole(specIdCheck)
    return role
end

--> CL: VALID PLAYER CHECK
local function ValidLogPlayer(flags, guid, name)
    if (not name) or (not guid) then
        return false
    end
    if not string.find(guid, "Player") then
        return false
    end
    if (bit_band(flags, 0x00000040) ~= 0 or weizPVP.Options.DEBUG) and bit_band(flags, 0x00000400) ~= 0 then
        return true
    end
    return false
end

--> CL: CREATE NEW PLAYER ENTRY IN DB
local function CLogNewPlayerFound(event, srcName, srcGUID, arg12)
    if weizPVP.PlayerDB[srcName] then
        return
    end
    if srcName and srcGUID then
        weizPVP.PlayerDB[srcName] = {}
        weizPVP.PlayerDB[srcName].Name = srcName
        weizPVP.PlayerDB[srcName].Timestamp = GetTime()
        weizPVP.PlayerDB[srcName].Estimated = true
        local levelNewPlayer
        local abilityType = strsub(event, 1, 5)
        if abilityType == "SWING" or abilityType == "SPELL" or abilityType == "RANGE" then
            levelNewPlayer = weizPVP.GetLevelFromSpellId(arg12) or 0
        else
            levelNewPlayer = 0
        end
        weizPVP.PlayerDB[srcName].Level = levelNewPlayer
        local class
        _, class = GetPlayerInfoByGUID(srcGUID)
        weizPVP.PlayerDB[srcName].Class = class
        local newRole = ClRoleCheck(arg12, class)
        weizPVP.PlayerDB[srcName].Role = newRole or nil
    end
end

--> CL: DEATH CHECK
local function CLDeath(fEvent, fGUID, fName)
    if weizPVP.PlayerActiveCache[fGUID] and fEvent == "UNIT_DIED" then
        weizPVP.PlayerActiveCache[fGUID].Dead = true
        weizPVP.UpdatePlayerActiveCache(fName, nil, nil, true, nil, fGUID)
        return true
    end
    return false --> return false on no death seen
end

--> CL: AuraCheck
local hpriest = {
    [27827] = GetSpellInfo(27827),
    [20711] = GetSpellInfo(20711),
    [126094] = GetSpellInfo(126094)
}
local stealthIDs = {
    [1784] = true,
    [115834] = true,
    [114018] = true,
    [11327] = true,
    [5215] = true,
    [66] = true,
    [156136] = true,
    [113862] = true
}
local stealthNames = {
    ["Stealth"] = true,
    ["Prowl"] = true
}
local function AuraCheck(spellId, spellName, srcGUID, srcName)
    --> Check for STEALTH match
    local stealthFound = false
    if stealthNames[spellName] then
        stealthFound = true
    elseif stealthIDs[spellName] then
        stealthFound = true
    end
    --> STEALTH FOUND?
    if stealthFound == true then
        if weizPVP.Options.StealthAlert.Enabled == true then
            local _, _, icon = GetSpellInfo(spellId)
            --> Set Class Color
            local classColoredName = WrapTextInColorCode(srcName, select(4, GetClassColor(weizPVP.PlayerDB[srcName].Class)))
            weizPVP.ShowStealthAlert(spellName, classColoredName, icon)
        end
        weizPVP.UpdatePlayerActiveCache(srcName, nil, true, false, nil, srcGUID)
        return true
    end
    --> Redemption Check
    if hpriest[spellId] then
        weizPVP.UpdatePlayerActiveCache(srcName, nil, nil, true, "HEALER", srcGUID)
        return true --> Return after updating player cache
    end
    return false
end

local string_find = string.find
local function CheckClSrcForRealm(srcName)
    if not srcName then
        return
    end
    if string_find(srcName, "-") then
        return srcName
    else
        return srcName .. "-" .. NS.PlayerRealm
    end
end

--> EVENT: COMBAT LOG
function weizPVP:CombatLogEvent()
    local _, event, _, srcGUID, srcName, srcFlags, _, dstGUID, dstName, dstFlags, _, spellId, spellName, _, _, _ = CombatLogGetCurrentEventInfo()
    --> IS VALID SOURCE?
    if ValidLogPlayer(srcFlags, srcGUID, srcName) then
        srcName = CheckClSrcForRealm(srcName)
        --> Check if unit is not in DB yet
        if not self.PlayerDB[srcName] then
            CLogNewPlayerFound(event, srcName, srcGUID, spellId)
        end
        --> Check role
        if self.PlayersOnBars[srcGUID] and self.PlayerActiveCache[srcGUID] then
            local role = ClRoleCheck(spellId, self.PlayerDB[srcName].Class)
            if role ~= nil then
                if self.PlayerDB[srcName].Role ~= role or not self.PlayerActiveCache[srcGUID].RoleFound then
                    self.PlayerDB[srcName].Role = role
                    self.PlayerActiveCache[srcGUID].RoleFound = true
                end
            end
        end
        --> AuraCheck
        if event == "SPELL_AURA_APPLIED" and srcGUID == dstGUID then
            local matchFound = AuraCheck(spellId, spellName, srcGUID, srcName)
            if matchFound then
                return --> Return after updating player cache
            end
        end
        weizPVP.UpdatePlayerActiveCache(srcName, nil, nil, nil, nil, srcGUID)
    end
    --> VALID DST; CHECK DEAD
    if ValidLogPlayer(dstFlags, dstGUID, dstName) then
        CLDeath(event, dstGUID, dstName)
    end
end

--|> UNIT EVENTS
-------------------------------------------------------------------------------------------
--> EVENT: MOUSEOVER
function weizPVP:PlayerMouseoverEvent()
    if UnitExists("mouseover") then
        self.GetUnitData("mouseover")
    end
end

--> EVENT: TARGET CHANGED
function weizPVP:PlayerTargetEvent()
    --> update player info
    self.GetUnitData("target")
    self.Crosshair.NewTarget() --: Crosshair->New Target
    self.ChangeTargetIcon() --> Refresh target icon from player bars
end

--> EVENT: NAMEPLATE ADDED
function weizPVP:NameplateAdded(_, unit)
    self.Crosshair.NameplateAdded(unit)
    --> Update time and stealth
    self.GetUnitData(unit)
    self.CurrentNameplates[UnitGUID(unit)] = true
end

--> EVENT: NAMEPLATE REMOVED
function weizPVP:NameplateRemoved(_, unit)
    self.Crosshair.NameplateRemoved(unit)
    self.GetUnitData(unit)
    self.CurrentNameplates[UnitGUID(unit)] = nil
end

--> EVENT: HEALTH
function weizPVP:UnitHealthEvent(_, unit)
    local GUID = UnitGUID(unit) or nil
    --> Check if player is already on bars
    if self.PlayersOnBars[GUID] and self.PlayerActiveCache[GUID] then
        local newhealth = (UnitHealth(unit) / UnitHealthMax(unit) * 100) or nil
        local dead = UnitIsDeadOrGhost(unit)
        --> If valid dead value that is different that previously stored; update.
        if dead ~= nil and dead ~= self.PlayerActiveCache[GUID].Dead then
            self.PlayerActiveCache[GUID].Dead = dead
        end
        if newhealth ~= nil and newhealth ~= self.PlayerActiveCache[GUID].Health then
            self.PlayerActiveCache[GUID].Health = newhealth
        end
    end
end

--> EVENT: UNIT TARGET
function weizPVP:UnitTargetEvent(_, unit)
    if self.IsUnitValidForTracking(unit) then
        --> Update for unit and their target
        self.GetUnitData(unit)
    end
end

--|> COMBAT EVENTS
-------------------------------------------------------------------------------------------
--> EVENT: ENTERING COMBAT
function weizPVP:EnteringCombat()
    self.SetStatusBarMessage("|cFFF4564D+++ IN COMBAT +++|r", false)
end

--> EVENT: LEAVING COMBAT
function weizPVP:LeavingCombat()
    self.SetStatusBarMessage("|cFF53F493--- OUT OF COMBAT ---|r", true, 3)
    self.ManageListTimeouts()
end

--|> PLAYER UPDATES
--> EVENT: PLAYER LEVELED UP
function weizPVP:PlayerLevelUp()
    self.Player.Level = UnitLevel("player") --> Update player level
end

--|> WOW CLIENT UPDATES
--> UPDATE DISPLAY DATA
--: Runs on PLAYER_LOGIN event
function weizPVP:UpdateDisplayData()
    XEN.DisplayUpdated()
    self.Display.Width = XEN._display.ScreenWidth
    self.Display.Height = XEN._display.ScreenHeight
    self.Display.UIScale = XEN._display.UIScale
    self.Display.UiMultiplier = XEN._display.UiMultiplier
    self.Display.EffectiveScale = XEN._display.EffectiveScale
    self.Display.DPIScale = XEN._display.DPIScale
    self.Display.xFactor = XEN._display.xFactor
    self.Display.yFactor = XEN._display.yFactor
end

--|> PULSE (1 second)
-------------------------------------------------------------------------------------------
local pulseTimeoutCount = 1
local function PulseTimeoutUpdate()
    if pulseTimeoutCount == 2 then
        weizPVP.ManageListTimeouts()
        pulseTimeoutCount = 1
    else
        pulseTimeoutCount = pulseTimeoutCount + 1
    end
end

local pulseFpsCount = 1
local function PulseFpsUpdate()
    if pulseFpsCount == 5 then
        weizPVP.Display.CurrentFPS = GetFramerate()
        pulseFpsCount = 1
    else
        pulseFpsCount = pulseFpsCount + 1
    end
end

--> Update pulsed functions
function weizPVP.PulseEvent()
    PulseFpsUpdate()
    PulseTimeoutUpdate()
end

--|> Phase Detection
-------------------------------------------------------------------------------------------
local time = time
local connectedRealms = GetAutoCompleteRealms() or nil
local RealmFound = false

local function CheckRealm()
    RealmFound = false
    connectedRealms = GetAutoCompleteRealms()
    if connectedRealms then
        tDeleteItem(connectedRealms, NS.PlayerRealm)
        if #connectedRealms > 0 then
            RealmFound = true
        end
    end
    if RealmFound == false then
        weizPVP.PrintAddonMessage("|cffe32dff~PHASING~|r")
        weizPVP.SetStatusBarMessage("|cffe32dff~PHASING~|r", true, 4)
    end
end

local total = 0
local shifted = false
local ShiftTime = time()
local NextShiftTime = ShiftTime + 1
local function phaseTimer(_, elapsed)
    total = total + elapsed
    if total >= 0.1 then
        if weizPVP.Options.Alerts.PhasingChat then
            if not GarrisonLandingPageMinimapButton:IsShown() then
                ShiftTime = time()
                NextShiftTime = ShiftTime + 1
                if shifted == false then
                    CheckRealm()
                    shifted = true
                end
            end
            if shifted == true then
                if time() >= NextShiftTime then
                    shifted = false
                end
            end
            total = 0
        end
    end
end

local f = CreateFrame("frame")
f:SetScript("OnUpdate", phaseTimer)
